#!/bin/sh
#
# Service management functions specific to FreeBSD.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
actFreebsdServiceStart () {
	if actRun service "$1" enabled >/dev/null 2>&1; then
		actRun service "$1" start >/dev/null
		cmdExitStatus=$?
	else
		actRun service "$1" onestart >/dev/null
		cmdExitStatus=$?
	fi
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
actFreebsdServiceStop () {
	actRun service "$1" stop >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
actFreebsdServiceRestart () {
	if actRun service "$1" enabled >/dev/null 2>&1; then
		actRun service "$1" restart >/dev/null
		cmdExitStatus=$?
	else
		actRun service "$1" onerestart >/dev/null
		cmdExitStatus=$?
	fi
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
actFreebsdServiceTryRestart () {
	if actRun service "$1" status >/dev/null 2>&1; then
		if actRun service "$1" enabled >/dev/null 2>&1; then
			actRun service "$1" restart >/dev/null
			cmdExitStatus=$?
		else
			actRun service "$1" onerestart >/dev/null
			cmdExitStatus=$?
		fi
		actDebug "command exit status: ${cmdExitStatus}"
	else
		actDebug "service not running, so no restart needed"
		cmdExitStatus=0
	fi
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
actFreebsdServiceReload () {
	if actRun service "$1" oneextracommands | tr ' ' '\n' | grep -Fqx 'reload'; then
		actRun service "$1" reload >/dev/null
		cmdExitStatus=$?
	else
		actError "UNSUPPORTED-ACTION-FOR-SERVICE" "${action}" "$1"
		return 2
	fi
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start on boot.
actFreebsdServiceEnable () {
	actRun service "$1" enable >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start on boot.
actFreebsdServiceDisable () {
	actRun service "$1" disable >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start on boot and cannot be started.
actFreebsdServiceMask () {
	# Service masking is not supported on FreeBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Unmask service $1 so it can be started and may start on boot.
actFreebsdServiceUnmask () {
	# Service masking is not supported on FreeBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Report "true" or "false" depending on whether service $1 exists.
actFreebsdServiceIsKnown () {
	actRun service -r | awk -F '/' '{print $NF}' | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
actFreebsdServiceIsRunning () {
	actRun service "$1" status >/dev/null 2>&1 && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled.
actFreebsdServiceIsEnabled () {
	actRun service "$1" enabled >/dev/null && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
actFreebsdServiceIsMasked () {
	# Service masking is not supported on FreeBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
actFreebsdServiceListAll () {
	actRun service -r | awk -F '/' '{print $NF}' | sort -u | \
	{
	while read -r serviceName; do
		test -n "${serviceName}" || continue

		bootStatus="disabled"
		actRun service "${serviceName}" enabled >/dev/null 2>&1 && bootStatus="enabled"

		activeStatus="inactive"
		actRun service "${serviceName}" status >/dev/null 2>&1 && activeStatus="active"

		serviceDescription="$(actRun service "${serviceName}" describe | tr '\t\n' '  ')"
		test -n "${serviceDescription}" || serviceDescription="-"

		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${bootStatus}" "${activeStatus}" "${serviceDescription}"
	done
	}

	return 0
}
