#!/bin/sh
#
# OpenBSD-specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all OpenBSD package actions - set "packageManagerFlags"
# according to whether interactive mode should be used; and set
# "packageListArgs" to the sanitised arguments.
#
actOpenbsdPackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments according to whether interactive mode is
	# enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="-I"

	# Sanitise the input arguments.
	packageListArgs="$(actSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
actOpenbsdPackageInstall () {
	actRun "${packageManagerCommand}" ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
actOpenbsdPackageUpgrade () {
	actRun "${packageManagerCommand}" -u ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
actOpenbsdPackageDowngrade () {
	actRun "${packageManagerCommand}" -D downgrade ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $* (actually ${packageListArgs}).
actOpenbsdPackageRemove () {
	actRun pkg_delete ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
actOpenbsdPackageReinstall () {
	actRun "${packageManagerCommand}" -u -D installed ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
actOpenbsdPackageInfo () {
	actRun pkg_info -q -c -f ${packageListArgs} \
	| awk -v OFS="\r" '
/^$/ && name~/[A-Za-z]/ { print name,arch,version,installed,vendor,"tgz",summary; name=""; summary="?" }
/^@name/ {
  name=$2
  sub(/-[^-]+$/,"",name)
  version=$2
  sub(/^.*-/,"",version)
  arch="?"
  pkgInfoDir=$2
  gsub(/[^0-9A-Za-z,_.-]/,"",pkgInfoDir)
  cmd="stat -f %m /var/db/pkg/" pkgInfoDir " 2>/dev/null"
  cmd | getline installed
  close(cmd)
  if (installed=="")
    installed=0
  vendor="?"
}
/^@arch/ { arch=$2 }
/^@signer/ { vendor=$2 }
/^[0-9A-Za-z]/ && name!~/[A-Za-z]/ { summary=$0 }
' \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $1 (actually ${packageListArgs}).
actOpenbsdPackageFiles () {
	queryOutput="$(actRun pkg_info -q -L ${packageListArgs})"
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	printf "%s\n" "${queryOutput}" | grep -E .
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
actOpenbsdPackageOwningFile () {
	queryOutput=$(actRun pkg_info -q -E "$1")
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sed 's,-[^-]*$,,' | sort -u
	return 0
}

# Refresh the cache.
actOpenbsdPackageCacheUpdate () {
	# This is unsupported on OpenBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Clean all caches.
actOpenbsdPackageCacheClean () {
	# This is unsupported on OpenBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# List packages ready to upgrade, one per line, tab separated columns -
# package, architecture, target version, repository, type.
actOpenbsdPackageUpgradeList () {
	upgradeOutput="$(LC_ALL=C actRun pkg_add -s -m -u | tr '\r' '\n' | awk '$1~/->/ && $NF=="ok" {print $1}')"
	upgradePackageList="$(printf "%s\n" "${upgradeOutput}" | sed 's,->.*$,,;s,-[^-]*$,,')"
	test -n "${upgradePackageList}" || return 0

	packageListArgs="${upgradePackageList}"
	packageQueryResult="$(actOpenbsdPackageInfo ${upgradePackageList} 2>/dev/null | awk -F "\t" '{print $1,$2}')"

	printf "%s\n" "${upgradeOutput}" \
	| {
		while read -r packageName; do
			targetVersion="${packageName##*->}"
			targetVersion="${targetVersion%:}"
			packageName="${packageName%->*}"
			packageName="${packageName%-*}"
			packageArchitecture="$(printf "%s\n" "${packageQueryResult}" | awk -v "name=${packageName}" '$1==name {print $2}')"
			packageRepository="-"
			test -n "${packageArchitecture}" || packageArchitecture="?"
			printf "%s\t%s\t%s\t%s\t%s\n" "${packageName}" "${packageArchitecture}" "${packageRepository}" "${targetVersion}" "tgz"
		done
	}

	return 0
}

# Upgrade all packages.
actOpenbsdPackageUpgradeAll () {
	actRun "${packageManagerCommand}" -u ${packageManagerFlags}
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
actOpenbsdPackageListInstalled () {
	actOpenbsdPackageInfo
	return $?
}
