#!/bin/sh
#
# Service management functions specific to OpenBSD.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
actOpenbsdServiceStart () {
	if actRun rcctl get "$1" flags 2>/dev/null | grep -Fqx "NO"; then
		actRun rcctl -f start "$1" >/dev/null
		cmdExitStatus=$?
	else
		actRun rcctl start "$1" >/dev/null
		cmdExitStatus=$?
	fi
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
actOpenbsdServiceStop () {
	actRun rcctl stop "$1" >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
actOpenbsdServiceRestart () {
	if actRun rcctl get "$1" flags 2>/dev/null | grep -Fqx "NO"; then
		actRun rcctl -f restart "$1" >/dev/null
		cmdExitStatus=$?
	else
		actRun rcctl restart "$1" >/dev/null
		cmdExitStatus=$?
	fi
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
actOpenbsdServiceTryRestart () {
	if actRun rcctl ls started 2>/dev/null | grep -Fqx "$1"; then
		if actRun rcctl get "$1" flags 2>/dev/null | grep -Fqx "NO"; then
			actRun rcctl -f restart "$1" >/dev/null
			cmdExitStatus=$?
		else
			actRun rcctl restart "$1" >/dev/null
			cmdExitStatus=$?
		fi
		actDebug "command exit status: ${cmdExitStatus}"
	else
		actDebug "service not running, so no restart needed"
		cmdExitStatus=0
	fi
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
actOpenbsdServiceReload () {
	actRun rcctl reload "$1"
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start on boot.
actOpenbsdServiceEnable () {
	actRun rcctl enable "$1" >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start on boot.
actOpenbsdServiceDisable () {
	actRun rcctl disable "$1" >/dev/null
	cmdExitStatus=$?
	actDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start on boot and cannot be started.
actOpenbsdServiceMask () {
	# Service masking is not supported on OpenBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Unmask service $1 so it can be started and may start on boot.
actOpenbsdServiceUnmask () {
	# Service masking is not supported on OpenBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Report "true" or "false" depending on whether service $1 exists.
actOpenbsdServiceIsKnown () {
	actRun rcctl ls all | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
actOpenbsdServiceIsRunning () {
	actRun rcctl ls started | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled.
actOpenbsdServiceIsEnabled () {
	actRun rcctl ls on | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
actOpenbsdServiceIsMasked () {
	# Service masking is not supported on OpenBSD.
	actError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
actOpenbsdServiceListAll () {

	enabledServices="$(actRun rcctl ls on)"

	# Note that "rcctl ls started" includes rogue services (those that
	# are running even though they are disabled), so we don't need to
	# also do "rcctl ls rogue".
	runningServices="$(actRun rcctl ls started)"

	actRun rcctl ls all | sort -u | \
	{
	while read -r serviceName; do
		test -n "${serviceName}" || continue

		bootStatus="disabled"
		printf "%s\n" "${enabledServices}" | grep -Fqx "${serviceName}" && bootStatus="enabled"

		activeStatus="inactive"
		printf "%s\n" "${runningServices}" | grep -Fqx "${serviceName}" && activeStatus="active"

		# Service descriptions are unavailable on OpenBSD.
		serviceDescription="-"

		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${bootStatus}" "${activeStatus}" "${serviceDescription}"
	done
	}

	return 0
}
