#!/bin/sh
#
# System information gathering functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Output the value of parameter $2 in os-release file $1, or "-" if it is
# missing or empty.
#
actReadOsReleaseParameter () {
	osReleaseFile="$1"
	osReleaseParameter="$2"

	actDebug "actReadOsReleaseParameter: file=[$osReleaseFile], parameter=[$osReleaseParameter]"

	test -s "${osReleaseFile}" || { printf "%s\n" "-"; return; }

	# NB the newline after the "p" is required by FreeBSD sed.
	rawValue="$(sed -n "/^[[:space:]]*${osReleaseParameter}=/{s///;s/[[:space:]]*\$//;p
}" "${osReleaseFile}" 2>/dev/null)"
	actDebug "actReadOsReleaseParameter: rawValue=[${rawValue}]"
	test -n "${rawValue}" || { printf "%s\n" "-"; return; }

	case "${rawValue}" in
	"'"*)
		# Single-quoted value - strip quotes and just use the result.
		rawValue="$(printf "%s\n" "${rawValue}" | sed 's/^.//;s/.$//')"
		test -n "${rawValue}" || rawValue="-"
		printf "%s\n" "${rawValue}"
		return
		;;
	'"'*)
		# Double-quoted value - strip quotes and continue.
		rawValue="$(printf "%s\n" "${rawValue}" | sed 's/^.//;s/.$//')"
		;;
	esac

	# Process backslash escapes.
	rawValue="$(printf "%s\n" "${rawValue}" | sed 's/\\\(.\)/\1/g')"

	test -n "${rawValue}" || rawValue="-"
	printf "%s\n" "${rawValue}"
}

# Report key system information - machine ID hash (SHA256 in hex of "act-"
# plus the machine ID), hostname, DNS domain name, kernel name (such as
# "Linux" or "FreeBSD"), kernel version, architecture, system RAM size,
# system CPU count, IP addresses (comma-separated); and OS name, ID, version
# ID, build ID, similar OS types ("ID_LIKE"), user-friendly OS name.
#
# Any of these may be "-" (or 0 for RAM size and CPU count).
#
actSystemInfo () {

	# Machine ID, if any, hashed with SHA256 after prepending "act-".
	machineIdHash="-"
	machineIdFile="/etc/machine-id"
	machineIdRaw=""
	test -s "${machineIdFile}" || machineIdFile="/var/lib/dbus/machine-id"
	test -s "${machineIdFile}" && machineIdRaw="$(actRun cat "${machineIdFile}" 2>/dev/null)"
	test "${machineIdRaw}" = "uninitialized" && machineIdRaw=""
	if test "${machineIdRaw}" = "" && command -v sysctl >/dev/null 2>&1; then
		# On OpenBSD, use hw.uuid as a machine ID instead.
		machineIdRaw="$(actRun sysctl -n hw.uuid 2>/dev/null | sed -n 1p)"
	fi
	if test -n "${machineIdRaw}"; then
		sha256cmd="sha256sum"
		if ! command -v "${sha256cmd}" >/dev/null 2>&1; then
			sha256cmd="sha256"
			if ! command -v "${sha256cmd}" >/dev/null 2>&1; then
				command -v openssl >/dev/null 2>&1 && sha256cmd="openssl sha256 -r"
			fi
		fi
		if test -n "${sha256cmd}"; then
			# NB $sha256cmd intentionally unquoted, as it may contain arguments.
			machineIdHash="$(printf "%s%s\n" "act-" "${machineIdRaw}" | actRun ${sha256cmd} 2>/dev/null | awk '{print $1}')"
			test -n "${machineIdHash}" || machineIdHash="-"
		fi
	fi

	# Hostname.
	dnsHost="$(actRun hostname -s 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	test -n "${dnsHost}" || dnsHost="$(actRun uname -n 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	test -n "${dnsHost}" || dnsHost="-"

	# DNS domain name.
	dnsDomain=""
	if command -v dnsdomainname >/dev/null 2>&1; then
		dnsDomain="$(actRun dnsdomainname 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	elif command -v domainname >/dev/null 2>&1; then
		dnsDomain="$(actRun domainname 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	else
		dnsDomain="$(actRun hostname -d 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	fi
	test -n "${dnsDomain}" || dnsDomain="-"

	# Kernel name.
	kernelName="$(actRun uname -s 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	test -n "${kernelName}" || kernelName="-"

	# Kernel version.
	kernelVersion="$(actRun uname -r 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	test -n "${kernelVersion}" || kernelVersion="-"

	# System architecture.
	arch="$(actRun uname -m 2>/dev/null | tr '\r\t' '  ' | sed -n 1p)"
	test -n "${arch}" || arch="-"

	# Total bytes of RAM available to the OS.
	ramSize="0"
	if test -e /proc/meminfo; then
		ramSize="$(actRun awk '/^MemTotal:/ { print 1024*$2 }' /proc/meminfo 2>/dev/null | sed -n 1p)"
		test -n "${ramSize}" || ramSize="0"
	fi
	if test "${ramSize}" = "0" && command -v sysctl >/dev/null 2>&1; then
		ramSize="$(actRun sysctl -n hw.physmem 2>/dev/null | sed -n 1p)"
		test -n "${ramSize}" || ramSize="0"
	fi

	# Number of CPUs.
	cpuCount="0"
	if test -e /proc/cpuinfo; then
		cpuCount="$(actRun actRun grep -Ec ^processor /proc/cpuinfo 2>/dev/null)"
		test -n "${cpuCount}" || cpuCount="0"
	fi
	if test "${cpuCount}" = "0" && command -v sysctl >/dev/null 2>&1; then
		cpuCount="$(actRun sysctl -n hw.ncpu 2>/dev/null | sed -n 1p)"
		test -n "${cpuCount}" || cpuCount="0"
	fi

	# List of IP addresses.
	ipAddresses="-"
	if command -v ip >/dev/null 2>&1; then
		ipAddresses="$(LC_ALL=C actRun ip addr show | awk '$1~/^inet/ && /scope global/ { sub(/\/.*$/,"",$2); print $2 }' 2>/dev/null | tr '\n' ',' | sed 's/,$//')"
		test -n "${ipAddresses}" || ipAddresses="-"
	fi
	if test "${ipAddresses}" = "-" && command -v ifconfig >/dev/null 2>&1; then
		ipAddresses="$(LC_ALL=C actRun ifconfig | sed 's/ addr:/ /g' | awk '$1~/^inet/ { sub(/[\/%].*$/,"",$2); print $2 }' | grep -Fv -e '127.0.0.1' -e '::1' | tr '\n' ',' | sed 's/,$//')"
		test -n "${ipAddresses}" || ipAddresses="-"
	fi

	osName="-"		# NAME
	osId="-"		# ID
	osVersionId="-"		# VERSION_ID
	osBuildId="-"		# BUILD_ID
	osLike="-"		# ID_LIKE
	osCpeName="-"		# CPE_NAME
	osPrettyName="-"	# PRETTY_NAME

	# Find the location of os-release(5).
	osReleaseFile=""
	for candidateFile in /etc/os-release /usr/lib/os-release; do
		test -s "${candidateFile}" || continue
		osReleaseFile="${candidateFile}"
		break
	done

	# Note that we could just source the os-release file, but this means
	# executing the contents of a file completely outside of our
	# control.  Since this action is unlikely to be run very often, the
	# overhead of manually parsing the contents is worth the security
	# improvement it brings.

	if test -n "${osReleaseFile}"; then
		# systemd os-release file.
		osName="$(actReadOsReleaseParameter "${osReleaseFile}" "NAME")"
		osId="$(actReadOsReleaseParameter "${osReleaseFile}" "ID")"
		osVersionId="$(actReadOsReleaseParameter "${osReleaseFile}" "VERSION_ID")"
		osBuildId="$(actReadOsReleaseParameter "${osReleaseFile}" "BUILD_ID")"
		osLike="$(actReadOsReleaseParameter "${osReleaseFile}" "ID_LIKE")"
		osCpeName="$(actReadOsReleaseParameter "${osReleaseFile}" "CPE_NAME")"
		osPrettyName="$(actReadOsReleaseParameter "${osReleaseFile}" "PRETTY_NAME")"
	elif test -s /etc/redhat-release; then
		# Red Hat release file.
		redhatRelease="$(actRun cat /etc/redhat-release 2>/dev/null)"
		osName="$(printf "%s\n" "${redhatRelease}" | sed 's/ release.*$//' | cut -d '(' -f 1 | sed 's/[[:space:]]*$//')"
		test -n "${osName}" || osName="-"
		osId="rhel"
		printf "%s\n" "${redhatRelease}" | grep -Eq ^CentOS && { osId="centos"; osLike="rhel"; }
		osVersionId="$(printf "%s\n" "${redhatRelease}" | sed -n 's/^.* release //p' | cut -d '(' -f 1 | sed 's/[[:space:]]*$//')"
		test -n "${osVersionId}" || osVersionId="-"
		osPrettyName="$(printf "%s\n" "${redhatRelease}" | sed 's/ release / /')"
		test -n "${osPrettyName}" || osPrettyName="-"
	elif test -s /etc/motd && sed -n 1p /etc/motd 2>/dev/null | grep -Eq ^OpenBSD; then
		# Get some info from the OpenBSD MOTD first line.
		osName="OpenBSD"
		osId="openbsd"
		osVersionId="${kernelVersion}"
		osPrettyName="$(sed -n 1p /etc/motd | sed 's/ #.*$//' | tr '\r\t' '  ')"
		test -n "${osPrettyName}" || osPrettyName="-"
	fi

	# Output the collected information.
	printf "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" \
	  "${machineIdHash}" \
	  "${dnsHost}" \
	  "${dnsDomain}" \
	  "${kernelName}" \
	  "${kernelVersion}" \
	  "${arch}" \
	  "${ramSize}" \
	  "${cpuCount}" \
	  "${ipAddresses}" \
	  "${osName}" \
	  "${osId}" \
	  "${osVersionId}" \
	  "${osBuildId}" \
	  "${osLike}" \
	  "${osCpeName}" \
	  "${osPrettyName}"
}
